from prophet.plot import plot_plotly, plot_components_plotly
from prophet import Prophet
from cassandra.references.load_holidays import load_holidays
import pandas as pd


def prophet(df, name_date_column, name_target_column, window_start='', window_end='', national_holidays_abbreviation='IT', future_dataframe_periods=14, plot_prophet=True):

    # Read the CSV on holidays
    csv_holiday = load_holidays()

    # Select the holidays according to the country that interests me
    condition = (csv_holiday['country'] == national_holidays_abbreviation)
    holidays = csv_holiday.loc[condition, ['ds', 'holiday']]

    # Create a DF with the only two columns for Prophet
    prophet_df = df[[name_date_column, name_target_column]]

    # Rename the columns for Prophet
    prophet_df = prophet_df.rename(columns={name_date_column: 'ds', name_target_column: 'y'})

    # Instance and fit Prophet
    prophet_m = Prophet(weekly_seasonality=True, yearly_seasonality=True, daily_seasonality=True, holidays=holidays)
    prophet_m.add_seasonality(name='monthly', period=30.5, fourier_order=5)
    prophet_m.fit(prophet_df)

    future = prophet_m.make_future_dataframe(periods=future_dataframe_periods)

    forecast = prophet_m.predict(future)
    forecast[['ds', 'yhat', 'trend', 'yearly', 'monthly', 'weekly', 'daily', 'holidays', 'additive_terms',
              'multiplicative_terms']].tail()

    sub_prophet_df = forecast[['ds', 'trend', 'yearly', 'monthly', 'weekly', 'daily', 'holidays']]
    sub_prophet_df = sub_prophet_df.rename(columns={'ds': name_date_column})

    if plot_prophet:
        plot_components_plotly(prophet_m, forecast)

    df[name_date_column] = pd.to_datetime(df[name_date_column])
    sub_prophet_df[name_date_column] = pd.to_datetime(sub_prophet_df[name_date_column])

    full_df = pd.merge(df, sub_prophet_df, how='inner', on=name_date_column)

    if window_start:
        full_df.drop(full_df[full_df[name_date_column] < window_start].index, inplace=True)
    if window_end:
        full_df.drop(full_df[full_df[name_date_column] > window_end].index, inplace=True)


    return full_df







from cassandra.model.linear import linear
from cassandra.model.logLinear import logLinear
from cassandra.model.ridge import ridge


def nestedModel(df, X_columns, y_column, model_regression = 'linear', metric = ['rsq', 'nrmse', 'mape'], return_metric = False):
    metrics_values = {}
    name_model = 'Nested Model for ' + y_column

    X = df[X_columns]
    y = df[y_column]

    if model_regression == 'linear':
        if return_metric:
            result, model, metrics_values = linear(df, X, y, y_column, name_model, metric = metric, return_metric = return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric = metric, return_metric = return_metric)

    elif model_regression == 'logLinear':
        if return_metric:
            result, model, metrics_values = logLinear(df, X, y, y_column, name_model, metric=metric,
                                                   return_metric=return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric=metric, return_metric=return_metric)

    elif model_regression == 'ridge':
        if return_metric:
            result, model, metrics_values = ridge(df, X, y, y_column, name_model, metric=metric,
                                                   return_metric=return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric=metric, return_metric=return_metric)

    if metrics_values:
        return result, model, metrics_values
    else:
        return result, model



def mainNestedModel(sub_df, sub_X_columns, sub_y_column, X_columns, y_column, sub_model_regression = 'linear', sub_metric = ['rsq', 'nrmse', 'mape'], return_sub_metric = False, model_regression = 'linear', metric = ['rsq', 'nrmse', 'mape'], return_metric = False):
    metrics_values = {}
    if return_sub_metric == True:
        sub_result, sub_model, sub_metric_values = nestedModel(sub_df, sub_X_columns, sub_y_column, sub_model_regression, sub_metric, return_sub_metric)
    else:
        sub_result, sub_model = nestedModel(sub_df, sub_X_columns, sub_y_column, sub_model_regression, sub_metric)

    df = sub_result

    df[sub_y_column] = sub_result['prediction']
    X = df[X_columns]
    y = df[y_column]

    name_model = model_regression + ' for ' + y_column

    if model_regression == 'linear':
        if return_metric:
            result, model, metrics_values = linear(df, X, y, y_column, name_model, metric=metric,
                                                   return_metric=return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric=metric, return_metric=return_metric)

    elif model_regression == 'logLinear':
        if return_metric:
            result, model, metrics_values = logLinear(df, X, y, y_column, name_model, metric=metric,
                                                      return_metric=return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric=metric, return_metric=return_metric)

    elif model_regression == 'ridge':
        if return_metric:
            result, model, metrics_values = ridge(df, X, y, y_column, name_model, metric=metric,
                                                  return_metric=return_metric)
        else:
            result, model = linear(df, X, y, y_column, name_model, metric=metric, return_metric=return_metric)

    if metrics_values:
        return result, model, metrics_values, sub_result, sub_model, sub_metric_values
    else:
        return result, model, sub_result, sub_model









import statsmodels.api as sm


def ols(X, y, constant=True):
    if constant:
        X = sm.add_constant(X)

    model = sm.OLS(y, X)

    result = model.fit()
    print(result.summary())

    return result, model
